#!/usr/bin/perl

# Log file firewall reporter
# for use with iptables
# By Chris Travers
# chris@travelamericas.com

# this script assumes that the only log messages that contain kernel:, SRC=,
# and DST= are IPTable entries.
{
	if ($ARGV[0] eq "--month") {
		$month=$ARGV[1];
		$type = "m";
		$log_path=$ARGV[2];
		if (!$log_path){
			$log_path="/var/log/messages";
		}
		$match_str=$month;
	}
	if ($ARGV[0] eq "--day") {
		$month=$ARGV[1];
		$day=$ARGV[2];
		if ($day){ 
			$type="d";
			if ($day < 10){
				$day=" $day"; # adding a space in front
			}
		}
		else {
			die "Day field is required!";
		}
		if ($ARGV[2] > 31 || $ARGV[2] < 1 || $ARGV[2] !~ /^\d+$/ ){
				die "Day in invalid format!";
		}
		$log_path=$ARGV[3];
		if (!$log_path){
			$log_path="/var/log/messages";
		}
		$match_str="$month $day";
	}
	if (!$type) {
		print "fwreport --month|day month [day] [path/to/log] \n";
		print "day is required for daily reports.\n";
		print "default log file at /var/log/messages\n";
		die;
	}

}

open (LOG, $log_path) || die "Can't open logfile: $! \n";

print {STDERR} "Parsing log file.......\n";

while ($line=<LOG>){
	if ($line =~ /^$match_str/ 
		&& $line =~ /kernel:/
		&& $line =~ /MAC=/
		&& $line =~ /SRC=/
		&& $line =~ /DPT=/
	){
		@elements = split(/ +/, $line);
		$line_element{"Mon"}=$elements[0];
		$line_element{"Day"}=$elements[1];
		foreach $element (@elements){
			if ($element =~ /^SRC=/){
				@sub_elements = split(/=/, $element);
				$line_element{"ip_addr"} = $sub_elements[1];
			}
			if ($element =~ /^IN=/){
				@sub_elements = split(/=/, $element);
				$line_element {"iface"} = $sub_elements[1];
			}
			if ($element =~ /^DPT=/){
				@sub_elements = split (/=/, $element);
				$line_element {"port"} = $sub_elements[1];
			}
			if ($element =~ /^PROTO=/){
				@sub_elements = split (/=/, $element);
				$line_element {"proto"} = $sub_elements[1];
			}
		}
		$line_month=$line_element{"Mon"};
		$line_day=$line_element{"Day"};
		if ($master {"$line_day"}
			{$line_element{"proto"}}
			{$line_element{"port"}}
			{$line_element{"ip_addr"}}{"hits"}){

			++$master {"$line_day"}
				{$line_element{"proto"}}
				{$line_element{"port"}}
				{$line_element{"ip_addr"}}
				{"hits"};
		}
		else {
			$master {"$line_day"}
				{$line_element{"proto"}}
				{$line_element{"port"}}
				{$line_element{"ip_addr"}}
				{"hits"} = 1;
		}
		if (!$ip_addr{$line_element{"ip_addr"}}{"name"}){
			$ip=pack C4, split (/\./, $line_element{"ip_addr"});
			$name = gethostbyaddr ($ip, "2");      
			$ip_addr{$line_element{"ip_addr"}}{"name"}=$name;
		}
		$master{$line_day}{$line_element{"proto"}}
				{$line_element{"port"}}
				{$line_element{"ip_addr"}}{"name"} 
			= 
			$ip_addr{$line_element{"ip_addr"}}{"name"};
	}
} 
close (LOG);

print {STDERR} "Associating ports with services.....\n";
foreach $date (keys %master){
	@protos=keys(%{$master{$date}});
	foreach $proto (@protos){
		foreach $port (%{$master{$date}{$proto}}){
			$service=getservbyport ($port, lc ($proto));
			if (!$service){
				$service="Unknown";
			}
			$master{$date}{$proto}{$port}{"service"}=$service;
		}
	}
}

if ($type eq "m"){
        print {STDERR} "Generating monthly statistics.... \n";
	foreach $date (keys %master){
		foreach $proto (keys %{$master{$date}}){
			foreach $port (keys %{$master{$date}{$proto}}){
				@ip_addrs=keys(%{$master{$date}{$proto}
				       {$port}});
				if (!$master{"monthly"}{$proto}{$port}
				    {"service"}){
					$master{"monthly"}{$proto}{$port}
						{"service"} 
						=
						$master{$date}{$proto}{$port}
							{"service"};
				}
				foreach $ip_addr (@ip_addrs){
					$master{"monthly"}{$proto}{$port}
						{$ip_addr}{"hits"}
						+=
						$master{$date}{$proto}{$port}
							{$ip_addr}{"hits"};
					$master{monthly}{$proto}{$port}
						{$ip_addr}{"name"}
						=
						$master{$date}{$proto}{$port}
							{$ip_addr}{"name"};
				}
			}
		}
	}
}
print"=====================Firewall Log report==============================\n";
if ($type eq "m"){
	print"Monthly Edition for month of $month\n";
}
else {
	print "Daily edition for $month $day\n";
}
print "Generated by: FWReport $version\n";
print"======================================================================\n";


foreach $date (sort {sort_date($a, $b)} keys %master){
	if ($type eq "m"){
		if ($date eq "monthly"){
			print "Monthly Summary:\n";
			print "=====================\n";
		}
		else {
			print "$month $date :\n";
			print "========================\n";
		}
	}
    
	foreach $proto (sort keys %{$master{$date}}){
		print "==================================\n";
		print "Transport Level Protocol: $proto\n";
		foreach $port (sort {$a <=> $b} keys %{$master{$date}{$proto}}){
			if ($port <= 0){
				next;
			}
			print "===================================\n";
			print "Port number: " . $port . "\n";
			print "Used by service: " . $master{$date}{$proto}
				{$port}{"service"} . "\n";
			print "===================================\n";

			print "# Hits    Address            Name\n";
		    
REPLINE:		foreach $ip_addr (sort keys %{$master{$date}{$proto}
					  {$port}}){
				if ($ip_addr eq "service") {
				    next REPLINE;
				}
				$hits=$master{$date}{$proto}{$port}{$ip_addr}
					{"hits"};
				$name=$master{$date}{$proto}{$port}{$ip_addr}
					{"name"};
				if ($name eq "NULL"){
				        $name="";
				}
				$~="REPORT_LINE";
				write;
			}
	        }
	}
print"======================================================================\n";
}
    
sub sort_date {
	if ($_[0] eq "monthly") {
		return 1;
	}
	elsif ($_[1] eq "monthly") {
		retirn -1;
	}
	else {
		return ($_[0] <=> $_[1]);
	}
}
format REPORT_LINE =
@<<<<<<   @<<<<<<<<<<<<<<<   @<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
$hits, $ip_addr, $name
.
